# GMT Time language file for german mode [DE]
#
#	$Id: de.d,v 1.2 2004/08/05 04:05:19 pwessel Exp $
#-------------------------------------------------
# Monthnames, with 3-char and 1-char abbreviations
M	1	Januar		Jan	J
M	2	Februar		Feb	F
M	3	März		Mär	M
M	4	April		Apr	A
M	5	Mai		Mai	M
M	6	Juni		Jun	J
M	7	Juli		Jul	J
M	8	August		Aug	A
M	9	September	Sep	S
M	10	Oktober		Okt	O
M	11	November	Nov	N
M	12	Dezember	Dez	D
# Name and abbreviations for the word Week in this language
U	1	Woche		Wo	W
# Weeknames, with 2-char and 1-char abbreviations
# (List starts on Sunday regardless of TIME_WEEK_START)
W	1	Sonntag		So	S
W	2	Montag		Mo	M
W	3	Dienstag	Di	D
W	4	Mittwoch	Mi	M
W	5	Donnerstag	Do	D
W	6	Freitag		Fr	F
W	7	Samstag		Sa	S
