"ANNOT_MIN_ANGLE",
"ANOT_MIN_ANGLE",
"ANNOT_MIN_SPACING",
"ANOT_MIN_SPACING",
"ANNOT_FONT",
"ANOT_FONT",
"ANNOT_FONT_PRIMARY",
"ANOT_FONT_PRIMARY",
"ANNOT_FONT_SIZE",
"ANOT_FONT_SIZE",
"ANNOT_FONT_SIZE_PRIMARY",
"ANOT_FONT_SIZE_PRIMARY",
"ANNOT_OFFSET",
"ANOT_OFFSET",
"ANNOT_OFFSET_PRIMARY",
"ANOT_OFFSET_PRIMARY",
"ANNOT_FONT2",
"ANNOT_FONT_SECONDARY",
"ANOT_FONT2",
"ANOT_FONT_SECONDARY",
"ANNOT_FONT2_SIZE",
"ANOT_FONT2_SIZE",
"ANNOT_FONT_SIZE_SECONDARY",
"ANOT_FONT_SIZE_SECONDARY",
"ANNOT_OFFSET2",
"ANOT_OFFSET2",
"ANNOT_OFFSET_SECONDARY",
"ANOT_OFFSET_SECONDARY",
"BASEMAP_AXES",
"BASEMAP_FRAME_RGB",
"BASEMAP_TYPE",
"CHAR_ENCODING",
"COLOR_BACKGROUND",
"COLOR_FOREGROUND",
"COLOR_NAN",
"COLOR_IMAGE",
"COLOR_MODEL",
"D_FORMAT",
"DEGREE_FORMAT",
"DEGREE_SYMBOL",
"DOTS_PR_INCH",
"ELLIPSOID",
"FIELD_DELIMITER",
"FRAME_PEN",
"FRAME_WIDTH",
"GLOBAL_X_SCALE",
"GLOBAL_Y_SCALE",
"GRID_CROSS_SIZE",
"GRID_PEN",
"GRID_CROSS_SIZE_PRIMARY",
"GRID_PEN_PRIMARY",
"GRID_CROSS_SIZE_SECONDARY",
"GRID_FORMAT",
"GRID_PEN_SECONDARY",
"GRIDFILE_SHORTHAND",
"HISTORY",
"HEADER_FONT",
"HEADER_FONT_SIZE",
"HEADER_OFFSET",
"HSV_MIN_SATURATION",
"HSV_MAX_SATURATION",
"HSV_MIN_VALUE",
"HSV_MAX_VALUE",
"INPUT_CLOCK_FORMAT",
"INPUT_DATE_FORMAT",
"INTERPOLANT",
"IO_HEADER",
"N_HEADER_RECS",
"LABEL_FONT",
"LABEL_FONT_SIZE",
"LABEL_OFFSET",
"LINE_STEP",
"MAP_SCALE_FACTOR",
"MAP_SCALE_HEIGHT",
"MEASURE_UNIT",
"N_COPIES",
"OBLIQUE_ANNOTATION",
"OBLIQUE_ANOTATION",
"OUTPUT_CLOCK_FORMAT",
"OUTPUT_DATE_FORMAT",
"OUTPUT_DEGREE_FORMAT",
"PAGE_COLOR",
"PAGE_ORIENTATION",
"PAPER_MEDIA",
"PLOT_CLOCK_FORMAT",
"PLOT_DATE_FORMAT",
"PLOT_DEGREE_FORMAT",
"POLAR_CAP",
"PS_COLOR",
"PS_IMAGE_COMPRESS",
"PS_IMAGE_FORMAT",
"PS_LINE_CAP",
"PS_LINE_JOIN",
"PS_MITER_LIMIT",
"PS_VERBOSE",
"PSIMAGE_COMPRESS",
"PSIMAGE_FORMAT",
"TICK_LENGTH",
"TICK_PEN",
"TIME_FORMAT",
"TIME_FORMAT_PRIMARY",
"TIME_FORMAT_SECONDARY",
"TIME_IS_INTERVAL",
"TIME_INTERVAL_FRACTION",
"TIME_EPOCH",
"TIME_UNIT",
"TIME_SYSTEM",
"TIME_WEEK_START",
"TIME_LANGUAGE",
"UNIX_TIME",
"UNIX_TIME_POS",
"VECTOR_SHAPE",
"VERBOSE",
"WANT_EURO_FONT",
"WANT_LEAP_SECONDS",
"X_AXIS_LENGTH",
"Y_AXIS_LENGTH",
"X_ORIGIN",
"Y_ORIGIN",
"XY_TOGGLE",
"Y_AXIS_TYPE",
"Y2K_OFFSET_YEAR",
