#define GMTCASE_ANNOT_MIN_ANGLE	0
#define GMTCASE_ANOT_MIN_ANGLE	1
#define GMTCASE_ANNOT_MIN_SPACING	2
#define GMTCASE_ANOT_MIN_SPACING	3
#define GMTCASE_ANNOT_FONT	4
#define GMTCASE_ANOT_FONT	5
#define GMTCASE_ANNOT_FONT_PRIMARY	6
#define GMTCASE_ANOT_FONT_PRIMARY	7
#define GMTCASE_ANNOT_FONT_SIZE	8
#define GMTCASE_ANOT_FONT_SIZE	9
#define GMTCASE_ANNOT_FONT_SIZE_PRIMARY	10
#define GMTCASE_ANOT_FONT_SIZE_PRIMARY	11
#define GMTCASE_ANNOT_OFFSET	12
#define GMTCASE_ANOT_OFFSET	13
#define GMTCASE_ANNOT_OFFSET_PRIMARY	14
#define GMTCASE_ANOT_OFFSET_PRIMARY	15
#define GMTCASE_ANNOT_FONT2	16
#define GMTCASE_ANNOT_FONT_SECONDARY	17
#define GMTCASE_ANOT_FONT2	18
#define GMTCASE_ANOT_FONT_SECONDARY	19
#define GMTCASE_ANNOT_FONT2_SIZE	20
#define GMTCASE_ANOT_FONT2_SIZE	21
#define GMTCASE_ANNOT_FONT_SIZE_SECONDARY	22
#define GMTCASE_ANOT_FONT_SIZE_SECONDARY	23
#define GMTCASE_ANNOT_OFFSET2	24
#define GMTCASE_ANOT_OFFSET2	25
#define GMTCASE_ANNOT_OFFSET_SECONDARY	26
#define GMTCASE_ANOT_OFFSET_SECONDARY	27
#define GMTCASE_BASEMAP_AXES	28
#define GMTCASE_BASEMAP_FRAME_RGB	29
#define GMTCASE_BASEMAP_TYPE	30
#define GMTCASE_CHAR_ENCODING	31
#define GMTCASE_COLOR_BACKGROUND	32
#define GMTCASE_COLOR_FOREGROUND	33
#define GMTCASE_COLOR_NAN	34
#define GMTCASE_COLOR_IMAGE	35
#define GMTCASE_COLOR_MODEL	36
#define GMTCASE_D_FORMAT	37
#define GMTCASE_DEGREE_FORMAT	38
#define GMTCASE_DEGREE_SYMBOL	39
#define GMTCASE_DOTS_PR_INCH	40
#define GMTCASE_ELLIPSOID	41
#define GMTCASE_FIELD_DELIMITER	42
#define GMTCASE_FRAME_PEN	43
#define GMTCASE_FRAME_WIDTH	44
#define GMTCASE_GLOBAL_X_SCALE	45
#define GMTCASE_GLOBAL_Y_SCALE	46
#define GMTCASE_GRID_CROSS_SIZE	47
#define GMTCASE_GRID_PEN	48
#define GMTCASE_GRID_CROSS_SIZE_PRIMARY	49
#define GMTCASE_GRID_PEN_PRIMARY	50
#define GMTCASE_GRID_CROSS_SIZE_SECONDARY	51
#define GMTCASE_GRID_FORMAT	52
#define GMTCASE_GRID_PEN_SECONDARY	53
#define GMTCASE_GRIDFILE_SHORTHAND	54
#define GMTCASE_HISTORY	55
#define GMTCASE_HEADER_FONT	56
#define GMTCASE_HEADER_FONT_SIZE	57
#define GMTCASE_HEADER_OFFSET	58
#define GMTCASE_HSV_MIN_SATURATION	59
#define GMTCASE_HSV_MAX_SATURATION	60
#define GMTCASE_HSV_MIN_VALUE	61
#define GMTCASE_HSV_MAX_VALUE	62
#define GMTCASE_INPUT_CLOCK_FORMAT	63
#define GMTCASE_INPUT_DATE_FORMAT	64
#define GMTCASE_INTERPOLANT	65
#define GMTCASE_IO_HEADER	66
#define GMTCASE_N_HEADER_RECS	67
#define GMTCASE_LABEL_FONT	68
#define GMTCASE_LABEL_FONT_SIZE	69
#define GMTCASE_LABEL_OFFSET	70
#define GMTCASE_LINE_STEP	71
#define GMTCASE_MAP_SCALE_FACTOR	72
#define GMTCASE_MAP_SCALE_HEIGHT	73
#define GMTCASE_MEASURE_UNIT	74
#define GMTCASE_N_COPIES	75
#define GMTCASE_OBLIQUE_ANNOTATION	76
#define GMTCASE_OBLIQUE_ANOTATION	77
#define GMTCASE_OUTPUT_CLOCK_FORMAT	78
#define GMTCASE_OUTPUT_DATE_FORMAT	79
#define GMTCASE_OUTPUT_DEGREE_FORMAT	80
#define GMTCASE_PAGE_COLOR	81
#define GMTCASE_PAGE_ORIENTATION	82
#define GMTCASE_PAPER_MEDIA	83
#define GMTCASE_PLOT_CLOCK_FORMAT	84
#define GMTCASE_PLOT_DATE_FORMAT	85
#define GMTCASE_PLOT_DEGREE_FORMAT	86
#define GMTCASE_POLAR_CAP	87
#define GMTCASE_PS_COLOR	88
#define GMTCASE_PS_IMAGE_COMPRESS	89
#define GMTCASE_PS_IMAGE_FORMAT	90
#define GMTCASE_PS_LINE_CAP	91
#define GMTCASE_PS_LINE_JOIN	92
#define GMTCASE_PS_MITER_LIMIT	93
#define GMTCASE_PS_VERBOSE	94
#define GMTCASE_PSIMAGE_COMPRESS	95
#define GMTCASE_PSIMAGE_FORMAT	96
#define GMTCASE_TICK_LENGTH	97
#define GMTCASE_TICK_PEN	98
#define GMTCASE_TIME_FORMAT	99
#define GMTCASE_TIME_FORMAT_PRIMARY	100
#define GMTCASE_TIME_FORMAT_SECONDARY	101
#define GMTCASE_TIME_IS_INTERVAL	102
#define GMTCASE_TIME_INTERVAL_FRACTION	103
#define GMTCASE_TIME_EPOCH	104
#define GMTCASE_TIME_UNIT	105
#define GMTCASE_TIME_SYSTEM	106
#define GMTCASE_TIME_WEEK_START	107
#define GMTCASE_TIME_LANGUAGE	108
#define GMTCASE_UNIX_TIME	109
#define GMTCASE_UNIX_TIME_POS	110
#define GMTCASE_VECTOR_SHAPE	111
#define GMTCASE_VERBOSE	112
#define GMTCASE_WANT_EURO_FONT	113
#define GMTCASE_WANT_LEAP_SECONDS	114
#define GMTCASE_X_AXIS_LENGTH	115
#define GMTCASE_Y_AXIS_LENGTH	116
#define GMTCASE_X_ORIGIN	117
#define GMTCASE_Y_ORIGIN	118
#define GMTCASE_XY_TOGGLE	119
#define GMTCASE_Y_AXIS_TYPE	120
#define GMTCASE_Y2K_OFFSET_YEAR	121
