# GMT Time language file for SE (Swedish) mode [SE]
#
#	$Id: se.d,v 1.1 2002/02/25 01:44:40 pwessel Exp $
#-------------------------------------------------
# Monthnames, with 3-char and 1-char abbreviations
M	1	Januari		Jan	J
M	2	Februari	Feb	F
M	3	Mars		Mar	M
M	4	April		Apr	A
M	5	Maj		Maj	M
M	6	Juni		Jun	J
M	7	Juli		Jul	J
M	8	Augusti		Aug	A
M	9	September	Sep	S
M	10	Oktober		Okt	O
M	11	November	Nov	N
M	12	December	Dec	D
# Name and abbreviations for the word Week in this language
U	1	Vecka		Vec	V
# Weeknames, with 3-char and 1-char abbreviations
# (List starts on Sunday regardless of TIME_WEEK_START)
W	1	Söndag		Sön	S
W	2	Måndag		Mån	M
W	3	Tisdag		Tis	T
W	4	Onsdag		Ons	O
W	5	Torsdag		Tor	T
W	6	Fredag		Fre	F
W	7	Lördag		Lör	S
