#	$Id: gmtmedia.d,v 1.5 2001/12/24 18:23:20 pwessel Exp $
#
#	Media Definition File for GMT
#
#	This file must be placed in $GMTHOME/share
#
#	This file contains the description of page sizes for
#	various media not among the standard paper sizes in GMT
#	(which are listed in the gmtdefaults man page).
#	GMT will use all names in a case-insensitive way so that
#	Letter and leTTer both mean letter.
#
#	Two examples are provided.  Note that the lines are
#	commented out so they are not read at run-time.
#
#	Below is an example of a HP Designjet 755CM plotter
#	that is 36" wide.  Since the roll is infinitely
#	long we give it a height of 0.
#
#name		width(pts)	height(pts)
dj755		2592		0
iris		2412		3334
hp800		3024		0
#
#	This is an example of a homemade printer that only
#	takes square paper that are 300 points on each side
#Zsquare	300		300
