# GMT Time language file for NL (Dutch/Netherlands) mode [NL]
#
#       $Id: nl.d,v 1.1 2002/02/25 01:44:40 pwessel Exp $
#-------------------------------------------------
# Monthnames, with 3-char and 1-char abbreviations
M	1	Januari		Jan	J
M	2	Februari	Feb	F
M	3	Maart		Mar	M
M	4	April		Apr	A
M	5	Mei		Mei	M
M	6	Juni		Jun	J
M	7	Juli		Jul	J
M	8	Augustus	Aug	A
M	9	September	Sep	S
M	10	Oktober		Okt	O
M	11	November	Nov	N
M	12	December	Dec	D
# Name and abbreviations for the word Week in this language
U	1	Week		Wk	W
# Weeknames, with 3-char and 1-char abbreviations
# (List starts on Sunday regardless of TIME_WEEK_START)
W	1	Zondag		Zo	Z
W	2	Maandag		Ma	M
W	3	Dinsdag		Di	D
W	4	Woensdag	Wo	W
W	5	Donderdag	Do	D
W	6	Vrijdag		Vr	V
W	7	Zaterdag	Za	Z
