# GMT 时间语言文件 中华人民共和国 (简体中文) 模式 [CN1]
#
#	$Id: cn1.d,v 1.1 2004/07/14 18:41:29 pwessel Exp $
#-------------------------------------------------
# 月份,  3位 和 1位 缩写
M	1	一月		一	J
M	2	二月		二	F
M	3	三月		三	M
M	4	四月		四	A
M	5	五月		五	M
M	6	六月		六	J
M	7	七月		七	J
M	8	八月		八	A
M	9	九月		九	S
M	10	十月		十	O
M	11	十一月		十一	N
M	12	十二月		十二	D
# Week在这种语言的名字和缩写
U	1	星期		周	W
# 星期名字, 3位和1位缩写
# (从星期日开始列起,不考虑 TIME_WEEK_START)
W	1	星期日		周日	S
W	2	星期一		周一	M
W	3	星期二		周二	T
W	4	星期三		周三	W
W	5	星期四		周四	T
W	6	星期五		周五	F
W	7	星期六		周六	S
