# GMT Time language file for DK (danish) mode [DK]
#
#	$Id: dk.d,v 1.1 2002/02/25 01:44:40 pwessel Exp $
#-------------------------------------------------
# Monthnames, with 3-char and 1-char abbreviations
M	1	Januar		Jan	J
M	2	Februar		Feb	F
M	3	Marts		Mar	M
M	4	April		Apr	A
M	5	Maj		Maj	M
M	6	Juni		Jun	J
M	7	Juli		Jul	J
M	8	August		Aug	A
M	9	September	Sep	S
M	10	Oktober		Okt	O
M	11	November	Nov	N
M	12	December	Dec	D
# Name and abbreviations for the word Week in this language
U	1	Uge		Uge	U
# Weeknames, with 3-char and 1-char abbreviations
# (List starts on Sunday regardless of TIME_WEEK_START)
W	1	Søndag		Søn	S
W	2	Mandag		Man	M
W	3	Tirsdag		Tir	T
W	4	Onsdag		Ons	O
W	5	Torsdag		Tor	T
W	6	Fredag		Fre	F
W	7	Lørdag		Lør	L
